﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Drawing.Imaging;
using System.Runtime.InteropServices;

namespace Watermarking_Demo
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
            ofd = new OpenFileDialog();
            sfd = new SaveFileDialog();
            string filter = "Image | *.jpg;*.png;*.gif";
            ofd.Filter = filter;
            sfd.Filter = filter;
        }

        OpenFileDialog ofd;
        SaveFileDialog sfd;
        string image_path;
        Bitmap image, processed, display, watermark;

        private void addInvisibleWatermarkToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                processed = image;
                watermark = (Bitmap)Bitmap.FromFile(ofd.FileName);
                processed = processed.AddInvisibleWatermark(watermark);
                display = processed.ScaleImage(image_frame.Width, image_frame.Height);
                image_frame.Image = display;
            }
        }

        private void addVisibleWatermarkToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                processed = image;
                watermark = (Bitmap)Bitmap.FromFile(ofd.FileName);
                processed = processed.AddVisibleWatermark(watermark, 0.5);
                display = processed.ScaleImage(image_frame.Width, image_frame.Height);
                image_frame.Image = display;
            }
        }

        private void saveImageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            sfd.FileName = image_path;
            if (sfd.ShowDialog() == DialogResult.OK)
            {
                processed.Save(sfd.FileName);
            }
        }

        private void loadImageToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (ofd.ShowDialog() == DialogResult.OK)
            {
                image_path = ofd.FileName;
                image = (Bitmap)Bitmap.FromFile(image_path);
                processed = image;
                display = processed.ScaleImage(image_frame.Width, image_frame.Height);
                image_frame.Image = display;
            }
        }
    }

    public static class Ext
    {
        public static Bitmap ScaleImage(this Bitmap image, int width, int height)
        {
            float xratio = (float)width / image.Width;
            float yratio = (float)height / image.Height;
            float ratio = Math.Min(xratio, yratio);

            int w = (int)(image.Width * ratio);
            int h = (int)(image.Height * ratio);

            Bitmap result = new Bitmap(image, new Size(w, h));
            return result;
        }

        public static Bitmap AddVisibleWatermark(this Bitmap image, Bitmap watermark, double opacity)
        {
            int w = image.Width;
            int h = image.Height;

            BitmapData image_data = image.LockBits(
                new Rectangle(0, 0, w, h),
                ImageLockMode.ReadOnly,
                PixelFormat.Format24bppRgb);

            int bytes = image_data.Stride * image_data.Height;
            byte[] buffer = new byte[bytes];
            byte[] result = new byte[bytes];

            Marshal.Copy(image_data.Scan0, buffer, 0, bytes);
            image.UnlockBits(image_data);

            int ww = watermark.Width;
            int wh = watermark.Height;

            if (w < ww || h < wh)
            {
                watermark = watermark.ScaleImage(w, h);
                ww = watermark.Width;
                wh = watermark.Height;
            }

            BitmapData wmark_data = watermark.LockBits(
                new Rectangle(0, 0, ww, wh),
                ImageLockMode.ReadOnly,
                PixelFormat.Format24bppRgb);

            int wbytes = wmark_data.Stride * wmark_data.Height;
            byte[] wbuffer = new byte[wbytes];

            Marshal.Copy(wmark_data.Scan0, wbuffer, 0, wbytes);
            watermark.UnlockBits(wmark_data);

            result = buffer;
            for (int i = 0; i < ww; i++)
            {
                for (int j = 0; j < wh; j++)
                {
                    int wposition = i * 3 + j * wmark_data.Stride;
                    int position = i * 3 + j * image_data.Stride;
                    for (int k = 0; k < 3; k++)
                    {
                        result[position + k] = (byte)((1 - opacity) * result[position + k] + opacity * wbuffer[wposition + k]);
                    }
                }
            }

            Bitmap res_img = new Bitmap(w, h);
            BitmapData res_data = res_img.LockBits(
                new Rectangle(0, 0, w, h),
                ImageLockMode.WriteOnly,
                PixelFormat.Format24bppRgb);
            Marshal.Copy(result, 0, res_data.Scan0, bytes);
            res_img.UnlockBits(res_data);
            return res_img;
        }

        public static Bitmap AddInvisibleWatermark(this Bitmap image, Bitmap watermark)
        {
            int w = image.Width;
            int h = image.Height;

            BitmapData image_data = image.LockBits(
                new Rectangle(0, 0, w, h),
                ImageLockMode.ReadOnly,
                PixelFormat.Format24bppRgb);

            int bytes = image_data.Stride * image_data.Height;
            byte[] buffer = new byte[bytes];
            byte[] result = new byte[bytes];

            Marshal.Copy(image_data.Scan0, buffer, 0, bytes);
            image.UnlockBits(image_data);

            int ww = watermark.Width;
            int wh = watermark.Height;

            if (w < ww || h < wh)
            {
                watermark = watermark.ScaleImage(w, h);
                ww = watermark.Width;
                wh = watermark.Height;
            }

            BitmapData wmark_data = watermark.LockBits(
                new Rectangle(0, 0, ww, wh),
                ImageLockMode.ReadOnly,
                PixelFormat.Format24bppRgb);

            int wbytes = wmark_data.Stride * wmark_data.Height;
            byte[] wbuffer = new byte[wbytes];

            Marshal.Copy(wmark_data.Scan0, wbuffer, 0, wbytes);
            watermark.UnlockBits(wmark_data);

            result = buffer;
            for (int i = 0; i < ww; i++)
            {
                for (int j = 0; j < wh; j++)
                {
                    int wposition = i * 3 + j * wmark_data.Stride;
                    int position = i * 3 + j * image_data.Stride;
                    for (int k = 0; k < 3; k++)
                    {
                        result[position + k] = (byte)((4 * result[position + k]) / 4 + wbuffer[wposition + k] / 64);
                    }
                }
            }

            Bitmap res_img = new Bitmap(w, h);
            BitmapData res_data = res_img.LockBits(
                new Rectangle(0, 0, w, h),
                ImageLockMode.WriteOnly,
                PixelFormat.Format24bppRgb);
            Marshal.Copy(result, 0, res_data.Scan0, bytes);
            res_img.UnlockBits(res_data);
            return res_img;
        }
    }
}
